/****************************************************************************
**
** Copyright (C) 2008 Andrey Rijov <ANDron142@yandex.ru>
**
** This file is part of AQEMU.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor,
** Boston, MA  02110-1301, USA.
**
****************************************************************************/

#include <QUuid>
#include <QDir>
#include <QSettings>
#include <QCloseEvent>
#include <QLineEdit>
#include <QFileDialog>
#include <QInputDialog>
#include <QMessageBox>

#include "Utils.h"
#include "Ports_Tab_Widget.h"
#include "Emulator_Control_Window.h"
#include "System_Info.h"

Emulator_Control_Window::Emulator_Control_Window( QWidget *parent )
	: QMainWindow( parent )
{
	ui.setupUi( this );
	Mon_Win = new Monitor_Window();
	
	QObject::connect( Mon_Win, SIGNAL(Closing_Win()),
					  this, SLOT(Mon_Win_Closed()) );
	
	AQDebug( "Emulator_Control_Window::Emulator_Control_Window( QWidget *parent )",
			 "Created" );
}

void Emulator_Control_Window::Mon_Win_Closed()
{
	Cur_VM->Set_Monitor_Mode( false );
}

void Emulator_Control_Window::closeEvent( QCloseEvent *event )
{
	Mon_Win->hide();
	event->accept();
}

void Emulator_Control_Window::Set_Current_VM( Virtual_Machine *vm )
{
	if( vm == NULL ) return;
	
	Cur_VM = vm;
	
	this->setWindowTitle( tr("Emulator Control") + " (" + Cur_VM->Get_Machine_Name() + ")" );
	
	// USB
	for( int ix = 0; ix < Cur_VM->Get_USB_Ports_Count(); ++ix )
	{
		QAction *new_act = new QAction( Cur_VM->Get_USB_Port(ix).Get_Product_Name(), ui.menuDisconnect );
		new_act->setData( Cur_VM->Get_USB_Port(ix).Get_ID_Line() );
		
		connect( new_act, SIGNAL(triggered()), this, SLOT(Delete_USB_From_VM()) );
		
		ui.menuDisconnect->addAction( new_act );
	}
}

void Emulator_Control_Window::on_actionSave_Screenshot_triggered()
{
	QSettings settings;
	
	if( settings.value("Use_Screenshots_Folder", "no").toString() == "yes" )
	{
		if( ! settings.value("Screenshot_Folder_Path", "").toString().isEmpty() )
		{
			// create unique file name
			QString unic_name = QUuid::createUuid();
			unic_name = unic_name.mid( 25, 12 );
			
			// save screenshot
			Cur_VM->Take_Screenshot( settings.value("Screenshot_Folder_Path", "").toString() +
					unic_name + "." + settings.value( "Screenshot_Save_Format", "PNG" ).toString() );
		}
	}
	else
	{
		AQGraphic_Warning( "Warning!",
						   "Shared Screenshots Folder Not Enabled!\nPlease Enter Name for New Screenshot." );
		on_actionSave_Screenshot_As_triggered();
	}
}

void Emulator_Control_Window::on_actionSave_Screenshot_As_triggered()
{
	QString file_name = QFileDialog::getSaveFileName( this, tr("Save Screenshot As..."),
													  QDir::homePath(), tr("All Files (*)") );
	
	if( file_name.isEmpty() )
	{
		AQWarning( "void Emulator_Control_Window::on_actionSave_Screenshot_As_triggered()",
				   "File Name is Empty! Screenshot Cannot be Save!" );
		return;
	}
	else
	{
		Cur_VM->Take_Screenshot( file_name );
	}
}

void Emulator_Control_Window::on_actionSave_VM_triggered()
{
	Cur_VM->Save_VM_State();
}

void Emulator_Control_Window::on_actionSave_VM_State_As_triggered()
{
	bool ok = false;
	QString VM_State_Name = QInputDialog::getText( this, tr("Enter VM State Name:"),
			tr("Tag Name:"), QLineEdit::Normal, "", &ok );
	
	if( ! ok ) return;
	
	if( VM_State_Name.isEmpty() )
	{
		AQGraphic_Warning( tr("Warning"),
						   tr("State Name is Empty! VM State Cannot be Save!") );
		return;
	}
	else
	{
		Cur_VM->Save_VM_State( VM_State_Name );
	}
}

void Emulator_Control_Window::on_actionLoad_VM_State_triggered() // fix, state list...
{
	bool ok;
	QString VM_State_Name = QInputDialog::getText( this, tr("Enter VM State Name:"),
			tr("Tag Name:"), QLineEdit::Normal, "", &ok );
	
	if( ok != true ) return;
	
	if( VM_State_Name.isEmpty() )
	{
		AQGraphic_Warning( tr("Warning"),
						   tr("Cannot Load VM State! State Name Invalid!") );
		return;
	}
	else
	{
		Cur_VM->Load_VM_State( VM_State_Name );
	}
}

void Emulator_Control_Window::on_actionCommit_triggered()
{
	emit Ready_Read_Command( "commit all" );
}

void Emulator_Control_Window::on_actionPause_VM_triggered()
{
	Cur_VM->Pause();
}

void Emulator_Control_Window::on_actionPower_Off_triggered()
{
	Cur_VM->Stop();
}

void Emulator_Control_Window::on_actionReset_VM_triggered()
{
	Cur_VM->Reset();
}

void Emulator_Control_Window::on_actionQEMU_Monitor_triggered()
{
	QObject::connect( Mon_Win,
					  SIGNAL(Command_Sended(QString)),Cur_VM,
					  SLOT(Execute_Emu_Ctl_Command(QString)) );
	
	Cur_VM->Set_Monitor_Mode( true );
	Mon_Win->show();
}

void Emulator_Control_Window::on_actionQuit_triggered()
{
	this->close();
}

void Emulator_Control_Window::on_actionFD0_dev_fd0_triggered()
{
	on_actionFD0_Eject_triggered();
	Set_Device( "fda", true, "/dev/fd0" );
}

void Emulator_Control_Window::on_actionFD0_dev_null_triggered()
{
	on_actionFD0_Eject_triggered();
	Set_Device( "fda", false, "/dev/null" );
}

void Emulator_Control_Window::on_actionFD0_Other_triggered()
{
	QString prev_file;
	
	if( Cur_VM->Get_FD0().Get_Host_Device() )
	{
		prev_file = Cur_VM->Get_FD0().Get_Host_File_Name();
	}
	else
	{
		prev_file = Cur_VM->Get_FD0().Get_Image_File_Name();
	}
	
	QString file_name = QFileDialog::getOpenFileName( this,
			tr("Open Device or Image File"),
			prev_file, tr("All Files (*)") );
	
	if( file_name.isEmpty() )
	{
		AQWarning( "void Emulator_Control_Window::on_actionFD0_Other_triggered()",
				   "Floppy Cannot be Changet! File Name is Empty!" );
		return;
	}
	else
	{
		on_actionFD1_Eject_triggered();
		Set_Device( "fda", false, file_name );
	}
}

void Emulator_Control_Window::on_actionFD0_Eject_triggered()
{
	emit Ready_Read_Command( "eject -f fda" );
}

void Emulator_Control_Window::on_actionFD1_dev_fd0_triggered()
{
	on_actionFD0_Eject_triggered();
	Set_Device( "fdb", true, "/dev/fd0" );
}

void Emulator_Control_Window::on_actionFD1_dev_null_triggered()
{
	on_actionFD0_Eject_triggered();
	Set_Device( "fdb", false, "/dev/null" );
}

void Emulator_Control_Window::on_actionFD1_Other_triggered()
{
	QString prev_file;
	
	if( Cur_VM->Get_FD1().Get_Host_Device() )
	{
		prev_file = Cur_VM->Get_FD1().Get_Host_File_Name();
	}
	else
	{
		prev_file = Cur_VM->Get_FD1().Get_Image_File_Name();
	}
	
	QString file_name = QFileDialog::getOpenFileName( this,
			tr("Open Device or Image File"),
			prev_file, tr("All Files (*)") );
	
	if( file_name.isEmpty() )
	{
		AQWarning( "void Emulator_Control_Window::on_actionFD1_Other_triggered()",
				   "Floppy Cannot be Changet! File Name is Empty!" );
		return;
	}
	else
	{
		on_actionFD1_Eject_triggered();
		Set_Device( "fdb", false, file_name );
	}
}

void Emulator_Control_Window::on_actionFD1_Eject_triggered()
{
	emit Ready_Read_Command( "eject -f fdb" );
}

void Emulator_Control_Window::on_actionCDROM_dev_cdrom_triggered()
{
	on_actionCDROM_Eject_triggered();
	Set_Device( "cdrom", true, "/dev/cdrom" );
}

void Emulator_Control_Window::on_actionCDROM_Other_triggered()
{
	QString prev_file;
	
	if( Cur_VM->Get_CD_ROM().Get_Host_Device() )
	{
		prev_file = Cur_VM->Get_CD_ROM().Get_Host_File_Name();
	}
	else
	{
		prev_file = Cur_VM->Get_CD_ROM().Get_Image_File_Name();
	}
	
	QString file_name = QFileDialog::getOpenFileName( this,
			tr("Open Device or Image File"),
			prev_file, tr("All Files (*)") );
	
	if( file_name.isEmpty() )
	{
		AQWarning( "void Emulator_Control_Window::on_actionCDROM_Other_triggered()",
				   "CD-ROM Cannot be Changet! File Name is Empty!" );
		return;
	}
	else
	{
		on_actionCDROM_Eject_triggered();
		Set_Device( "cdrom", false, file_name );
	}
}

void Emulator_Control_Window::on_actionCDROM_Eject_triggered()
{
	emit Ready_Read_Command( "eject -f cdrom" );
}

void Emulator_Control_Window::Delete_USB_From_VM()
{
	QAction *usb_item = qobject_cast<QAction*>( sender() );
	
	if( usb_item )
	{
		QList<QAction*> ac_list = ui.menuDisconnect->actions();
		
		for( int ix = 0; ix < ac_list.count(); ++ix )
		{
			if( usb_item->data().toString() == ac_list[ix]->data().toString() )
			{
				// Delete from Running VM
				QString bus_addr = Cur_VM->Get_USB_Bus_Address( usb_item->data().toString() );
				
				if( bus_addr.isEmpty() ) return;
				
				emit Ready_Read_Command( "usb_del " + bus_addr );
				
				// Delete item
				ui.menuDisconnect->removeAction( usb_item );
				
				return;
			}
		}
		
		// Cannot Find
		AQGraphic_Error( "void Emulator_Control_Window::Delete_USB_From_VM()",
						 tr("Error!"), tr("Cannot Find USB Device!"), false );
	}
	else
	{
		AQError( "void Emulator_Control_Window::Delete_USB_From_VM()",
				 "Cannot convert to QAction!" );
	}
}

void Emulator_Control_Window::Add_USB_To_VM()
{
	QAction *usb_item = qobject_cast<QAction*>( sender() );
	
	if( usb_item )
	{
		QList<QAction*> ac_list = ui.menuDisconnect->actions();
		
		for( int ix = 0; ix < ac_list.count(); ++ix )
		{
			// Unique value
			if( usb_item->data().toString() == ac_list[ix]->data().toString() )
			{
				AQWarning( "void Emulator_Control_Window::Add_USB_To_VM()",
						   "This is not unique value: " + usb_item->data().toString() );
				return;
			}
		}
		
		// Create Item for Disconnect Menu
		QAction *dis_act = new QAction( usb_item->text(), ui.menuDisconnect );
		dis_act->setData( usb_item->data() );
		
		connect( dis_act, SIGNAL(triggered()), this, SLOT(Delete_USB_From_VM()) );
		
		ui.menuDisconnect->addAction( dis_act );
		
		// Add To Running VM
		emit Ready_Read_Command( "usb_add host:" + usb_item->data().toString() );
	}
	else
	{
		AQError( "void Emulator_Control_Window::Add_USB_To_VM()",
				 "Cannot convert to QAction!" );
	}
}

void Emulator_Control_Window::on_actionUSB_Update_Device_List_triggered()
{
	// Get New USB Host List
	QList<VM_USB> tmp_list;
	
	if( ! System_Info::Get_All_Host_USB(tmp_list) )
	{
		AQGraphic_Warning( tr("Error!"),
						   tr("Cannot Get USB Information From System!") );
		return;
	}
	
	QList<QAction*> ac_list = ui.menuUSB_Connect->actions();
	
	// Delete Previous Menu (is avirable)
	if( ac_list.count() > 1 )
	{
		ui.menuUSB_Connect->clear();
		
		ui.menuUSB_Connect->addAction( ui.actionUSB_Update_Device_List );
		ui.menuUSB_Connect->addSeparator();
	}
	
	// Add items
	for( int ix = 0; ix < tmp_list.count(); ++ix )
	{
		QAction *new_act = new QAction( tmp_list[ix].Get_Product_Name(), ui.menuUSB_Connect );
		new_act->setData( tmp_list[ix].Get_ID_Line() );
		
		connect( new_act, SIGNAL(triggered()), this, SLOT(Add_USB_To_VM()) );
		
		ui.menuUSB_Connect->addAction( new_act );
	}
}

void Emulator_Control_Window::on_actionBy_Bus_Address_triggered()
{
	bool ok = false;
	
	QString bus_addr = QInputDialog::getText( this, tr("Disconnect USB"),
										  tr("Enter You Bus.Address VM USB Value"),
										  QLineEdit::Normal, "", &ok );
	
	if( ok && ! bus_addr.isEmpty() )
	{
		emit Ready_Read_Command( "usb_del " + bus_addr );
	}
}

void Emulator_Control_Window::on_actionUSB_Disconnect_All_Devices_triggered()
{
	QList<QAction*> ac_list = ui.menuDisconnect->actions();
	
	for( int dx = 0; dx < ac_list.count(); ++dx )
	{
		if( ac_list[dx]->data().isNull() == false &&
			ac_list[dx]->data().isValid() == true )
		{
			QString bus_addr = Cur_VM->Get_USB_Bus_Address( ac_list[dx]->data().toString() );
			
			if( bus_addr.isEmpty() ) return;
			
			emit Ready_Read_Command( "usb_del " + bus_addr );
		}
	}
	
	ui.menuDisconnect->clear();
	ui.menuDisconnect->addAction( ui.actionUSB_Disconnect_All_Devices );
	ui.menuDisconnect->addAction( ui.actionBy_Bus_Address );
	ui.menuDisconnect->addSeparator();
}

void Emulator_Control_Window::on_actionSerial_Null_triggered()
{
	emit Ready_Read_Command( "change serial \"/dev/null\"" );
}

void Emulator_Control_Window::on_action_Serial_ttyS0_triggered()
{
	emit Ready_Read_Command( "change serial \"/dev/ttyS0\"" );
}

void Emulator_Control_Window::on_actionSerial_Other_triggered()
{
	bool ok = false;
	
	QString ser_other = QInputDialog::getText( this, tr("Other..."),
			tr("Connect to:"), QLineEdit::Normal, "", &ok );
	
	if( ok && ! ser_other.isEmpty() )
	{
		emit Ready_Read_Command( "change serial \"" + ser_other + "\"" );
	}
}

void Emulator_Control_Window::on_actionParport_Null_triggered()
{
	emit Ready_Read_Command( "change parport \"/dev/null\"" );
}

void Emulator_Control_Window::on_actionParport_Other_triggered()
{
	bool ok = false;
	
	QString par_other = QInputDialog::getText( this, tr("Other..."),
			tr("Connect to:"), QLineEdit::Normal, "", &ok );
	
	if( ok && ! par_other.isEmpty() )
	{
		emit Ready_Read_Command( "change parport \"" + par_other + "\"" );
	}
}

void Emulator_Control_Window::on_actionCtrl_Alt_Delete_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-delete" );
}

void Emulator_Control_Window::on_actionOther_Keys_triggered()
{
	bool ok = false;
	
	QString keys = QInputDialog::getText( this, tr("Send Keys"),
			tr("Enter You Keys as ctrl-alt-delete or alt-f2"),
			QLineEdit::Normal, "", &ok );
	
	if( ok && ! keys.isEmpty() )
	{
		emit Ready_Read_Command( "sendkey " + keys );
	}
}

void Emulator_Control_Window::on_actionCtrl_Alt_Backspace_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-backspace" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F1_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f1" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F2_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f2" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F3_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f3" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F4_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f4" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F5_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f5" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F6_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f6" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F7_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f7" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F8_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f8" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F9_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f9" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F10_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f10" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F11_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f11" );
}

void Emulator_Control_Window::on_actionCtrl_Alt_F12_triggered()
{
	emit Ready_Read_Command( "sendkey ctrl-alt-f12" );
}

void Emulator_Control_Window::on_actionAbout_Emulator_Control_triggered()
{
	QMessageBox::information( this, tr("About Emulator Control..."),
							  tr("Emulator Control is a Tool for Managing QEMU in Runtime.") );
}

void Emulator_Control_Window::Set_Device( const QString &dev_name, bool host_dev, const QString &path )
{
	emit Ready_Read_Command( "change " + dev_name + " \"" + path + "\"" );
	
	if( dev_name == "fda" )
	{
		Cur_VM->Set_FD0( VM_Floppy(true, host_dev ? path : "",
						 ! host_dev ? path : "", host_dev) );
	}
	else if( dev_name == "fdb" )
	{
		Cur_VM->Set_FD1( VM_Floppy(true, host_dev ? path : "",
						 ! host_dev ? path : "", host_dev) );
	}
	else if( dev_name == "cdrom" )
	{
		Cur_VM->Set_CD_ROM( VM_CDROM(true, host_dev ? path : "",
							! host_dev ? path : "", host_dev) );
	}
	else
	{
		AQError( "void Emulator_Control_Window::Set_Device( const QString &dev_name, const QString &path )",
				 "Cannot Found Device: " + dev_name );
	}
}
